import * as React from 'react';
import {ChangeEvent} from 'react';
import {observer} from "mobx-react-lite";
import {DashboardComponentModel} from "../../../stores/DashboardComponentStore";
import {DashboardTargetModel} from "../../../stores/DashboardTargetStore";
import {CheckboxField, InputField, SelectField, useReadonlyContext, ModelDateField} from "@rx-suite/ui";
import {Parameter} from "../../../stores/Types";
import {t} from "@lingui/macro";
import {ComplexParameterField} from "./parameters/ComplexParameterField";
import {TimeLengthParameterField} from "./parameters/TimeLengthParameterField";
import moment from "moment";
import {ScoreCardRowParameter} from "./display_options/ScoreCardSettings";
import {runInAction} from "mobx";

export type ParameterFieldProps = {
    component: DashboardComponentModel,
    parameter: Parameter,
    row?: ScoreCardRowParameter,
    rowIndex?: number,
}

export const ParameterField: React.FunctionComponent<ParameterFieldProps> = observer(({
                                                                                          component,
                                                                                          parameter,
                                                                                          rowIndex,
                                                                                          row
                                                                                      }) => {
    const target = component.viewModel.dashboard_target as DashboardTargetModel;
    const readOnly = useReadonlyContext();
    const findValue = () => {
        if (typeof rowIndex !== 'number' || !row) return component.viewModel.parameters[parameter.id];
        return row[parameter.id];
    }
    const value = findValue();
    const findName = () => {
        if (typeof rowIndex !== 'number' || !row) return `parameters.${parameter.id}`;
        return `parameters.rows[${rowIndex}].${parameter.id}`;
    }
    const name = findName();
    const label = parameter.name;
    if (!target) {
        return null;
    }

    const hint = {
        title: parameter.name,
        children: <>
            {parameter.description}
            {(parameter.max || (parameter.max === 0)) &&
            <div>{t`Maximum Value:`}{parameter.max}</div>}
            {(parameter.min || (parameter.min === 0)) &&
            <div>{t`Minimum Value:`}{parameter.min}</div>}
            {(parameter.required) && <div>{t`This parameter is required`}</div>}
            {(parameter.value_list) && <div>{t`Available Values:`}
                <ul>
                    {parameter.value_list.map(v => <li key={v.value}>
                        {v.label}{v.description ? `: ${v.description}` : ''}
                    </li>)}
                </ul>
            </div>}</>
    }

    const handleOnChange = (event: ChangeEvent) => {
        component.handleChange(event);
        if (!row) {
            return;
        }
        const accessor = event.target?.name.split('.').pop();
        runInAction(() => {
            row[accessor] = event.target.value;
        })
    }
    const handleSelectChange = (name: string, value: any) => {
        component.handleSelectChange(name, value);
        if (!row) {
            return;
        }
        runInAction(() => {
            const accessor = name.split('.').pop();
            row[accessor] = value;
        })
    }
    const inputProps = {} as { min?: number, max?: number };
    switch (parameter.type) {
        case 'integer':
            if (parameter.min || (parameter.min === 0)) {
                inputProps.min = parameter.min;
            }
            if (parameter.max || (parameter.max === 0)) {
                inputProps.max = parameter.max;
            }
            return <InputField label={label}
                               name={name}
                               value={(value || value === 0) ? value : ''}
                               disabled={readOnly}
                               type={'number'}
                               inputProps={inputProps}
                               hint={hint}
                               onChange={handleOnChange}/>
        case 'boolean':
            return <CheckboxField label={label}
                                  name={name}
                                  checked={!!value}
                                  disabled={readOnly}
                                  hint={hint}
                                  onChange={handleOnChange}/>
        case 'string':
            if (parameter.value_list) {
                return <SelectField onChange={handleSelectChange}
                                    label={label}
                                    multiple={parameter.multiple}
                                    hint={hint}
                                    options={parameter.value_list.reduce((options: { label: string, options: { label: string, value: string }[] }[], valueDef) => {
                                        const groupName = valueDef.group || '';
                                        let group = options.find((o: { label: string }) => o.label === groupName);
                                        if (!group) {
                                            group = {
                                                label: groupName,
                                                options: []
                                            };
                                            options.push(group);
                                        }
                                        group.options.push({label: valueDef.label as string, value: valueDef.value});
                                        return options
                                    }, [])}
                                    name={name}
                                    disabled={readOnly}
                                    clearable
                                    value={value || (parameter.multiple ? [] : '')}/>
            } else {
                return <InputField label={label}
                                   name={name}
                                   value={(value || value === 0) ? value : ''}
                                   disabled={readOnly}
                                   hint={hint}
                                   onChange={handleOnChange}/>
            }
        case 'date':
            return <ModelDateField label={label}
                                   name={name}
                                   hint={hint}
                                   dateFormat={"YYYY-MM-DD"}
                                   timeFormat={false}
                                   value={(value || value === 0) ? moment(value).format('YYYY-MM-DD') : ''}
                                   disabled={readOnly} />
        case 'time':
            return <ModelDateField label={label}
                                   name={name}
                                   hint={hint}
                                   dateFormat={false}
                                   timeFormat={"HH:mm:ss"}
                                   value={(value || value === 0) ? moment(value).format('HH:mm:ss') : ''}
                                   disabled={readOnly} />
        case 'timelength':
            return <TimeLengthParameterField parameter={parameter} component={component} hint={hint}/>
        default:
            return <ComplexParameterField parameter={parameter} component={component}/>
    }
});
