import * as React from 'react';
import {FormEvent} from 'react';
import {observer} from "mobx-react-lite";
import {Conditional, FormProps, IfFalse, IfTrue, ModelForm, ModelInput, ModelSelect, PaddedPaper} from "@rx-suite/ui";
import {DashboardTargetStore} from "../../stores/DashboardTargetStore";
import {DashboardComponentModel} from "../../stores/DashboardComponentStore";
import {DashboardComponentParameters} from "../../components/dashboard_components/form/DashboardComponentParameters";
import {DashboardComponent} from "../../components/dashboard_components/renderers";
import {makeStyles} from "@material-ui/core/styles";
import {DashboardDisplayTypeOptions} from "../../components/dashboard_components/form/display_options/DashboardDisplayTypeOptions";
import {t} from "@lingui/macro";
import {Typography} from "@material-ui/core";
import moment from "moment";
import {DashboardComponentRunButton} from "../../components/dashboard_components/form/DashboardComponentRunButton";
import {TitleOptionsField} from "../../components/dashboard_components/form/display_options/TitleOptionsField";
import {DashboardComponentRunAs} from "../../components/dashboard_components/form/DashboardComponentRunAs";

type Option = { label: string, value: string };
type Group = { label: string, options: Option[] };

export type DashboardComponentFormProps = {
    afterSubmit?: (event: FormEvent, resp: any) => void,
    suppressRedirect?: boolean
} & FormProps

const useStyles = makeStyles({
    runData: {
        display: 'flex',
        alignItems: 'baseline'
    }
});

export const DashboardComponentsForm: React.FunctionComponent<DashboardComponentFormProps> = observer(({
                                                                                                           model,
                                                                                                           readOnly,
                                                                                                           suppressRedirect,
                                                                                                           afterSubmit
                                                                                                       }) => {
    const classes = useStyles();
    React.useEffect(() => {
        DashboardTargetStore.getAll();
    }, []);

    const targetOptions = DashboardTargetStore.models.reduce((options: Group[], target) => {
        let groupName = '';
        const uiHints = target.ui_hints as { category?: string } | undefined;
        if (target.module_name && uiHints && uiHints.category) {
            groupName = `${target.module_name} - ${uiHints.category}`
        } else {
            groupName = (target.module_name || '') as string
        }
        let group: Group | undefined = options.find((o: { label: string }) => o.label === groupName);
        if (!group) {
            group = {
                label: groupName,
                options: []
            };
            options.push(group);
        }
        group.options.push({label: target.name as string, value: target.getId()});
        group.options.sort((a, b) => a.label.localeCompare(b.label));
        return options
    }, []).sort((a, b) => a.label.localeCompare(b.label));
    React.useEffect(
        () => {
            const isTargetChanged = model.viewModel.dashboard_target_id != model.dashboard_target_id;
            if (model.isNew || isTargetChanged) {
                (model as DashboardComponentModel).resetParameters();
            }
            if (!isTargetChanged) {
                model.viewModel.reset();
            }
        },
        [model, model.viewModel.dashboard_target_id]
    );

    return (
        <PaddedPaper>
            <ModelForm model={model} readOnly={readOnly} suppressRedirect={suppressRedirect} afterSubmit={afterSubmit}
                       suppressWrapper>
                <ModelInput name={"name"}/>
                <ModelInput name={"desc"} multiline rowsMax={4}/>
                <TitleOptionsField component={model as DashboardComponentModel}/>
                <ModelInput name={"refresh_interval"} type={'number'} inputProps={{min: 5}}/>
                <ModelSelect name={"dashboard_target_id"} options={targetOptions}/>
                {model.viewModel.dashboard_target_id && <>
                    <DashboardComponentRunAs component={model as DashboardComponentModel}/>
                    <DashboardComponentParameters component={model as DashboardComponentModel}/>
                    <DashboardDisplayTypeOptions component={model as DashboardComponentModel}/>
                </>}
            </ModelForm>
            <Typography variant={'h4'}>{t`Data`}</Typography>
            {model.last_run_error && <div>{t`Last Run Error`}: {model.last_run_error}</div>}
            <Conditional condition={model.isNew}>
                <IfFalse>
                    <div className={classes.runData}>
                        <DashboardComponentRunButton component={model as DashboardComponentModel}/>
                        <Typography
                            variant={'body1'}>{t`Last Run`}: <Conditional condition={!!model.updated_at}>
                            <IfTrue>{moment(model.updated_at as Date).format('YYYY-MM-DD HH:mm:ss')}</IfTrue>
                            <IfFalse>{t`Never`}</IfFalse>
                        </Conditional></Typography>
                    </div>
                </IfFalse>
            </Conditional>
            <div>
                <DashboardComponent component={model as DashboardComponentModel}/>
            </div>
        </PaddedPaper>
    )
});
