import * as React from 'react';
import {useEffect} from 'react';
import {observer} from "mobx-react-lite";
import {
    ButtonField,
    Conditional,
    FormProps,
    IfFalse,
    IfTrue,
    ModelForm,
    ModelInput,
    SelectField
} from "@rx-suite/ui";
import {Grid, LinearProgress, makeStyles, Typography} from "@material-ui/core";
import {t, Trans} from "@lingui/macro";
import {AddLayoutMachineModal} from "../../components/lab_layout_edit/AddLayoutMachineModal";
import {LabLayoutComponentStore, LayoutComponentType} from "../../stores/LabLayoutComponentStore";
import {LabLayoutModel, LabLayoutResolutionOptions} from "../../stores/LabLayoutStore";
import {LayoutConfigComponentDetails} from "../../components/lab_layout_edit/LayoutConfigComponentDetails";
import {MachineStore} from "../../stores/MachineStore";
import {AddLayoutImageModal} from "../../components/lab_layout_edit/AddImageModal";
import {AddAlertGroupModal} from "../../components/lab_layout_edit/AddAlertGroupModal";
import MemoryIcon from '@material-ui/icons/Memory';
import {ErrorOutline, ImageRounded} from "@material-ui/icons";
import {MachineGroupStore} from "../../stores/MachineGroupStore";
import {LayoutEditViewRenderer} from "../../stores/object/LayoutEditViewRenderer";
import {AddStatusLegendButton} from "../../components/lab_layout_edit/AddStatusLegendButton";

export type LabLayoutFormProps = {
    model: LabLayoutModel;
} & FormProps

const useStyles = makeStyles({
    canvas: {
        touchAction: 'none',
        outline: 'none'
    }
});

// Convenience handler to get the base elements to manage a modal
export const useModalHandler = (): [boolean, ModalHandler, ModalHandler] => {
    const [open, setOpen] = React.useState(false);
    const handleClose: ModalHandler = React.useCallback(() => {
        setOpen(false)
    }, []);
    const handleOpen: ModalHandler = React.useCallback(() => {
        setOpen(true)
    }, []);
    return [open, handleClose, handleOpen];
}

type ModalHandler = () => void;

export const LabLayoutForm: React.FunctionComponent<LabLayoutFormProps> = observer(({model}) => {
    const classes = useStyles();

    MachineStore.getAll();
    MachineGroupStore.getAll();

    const [addMachineOpen, handleCloseAddMachine, handleOpenAddMachine] = useModalHandler();
    const [addImageOpen, handleCloseAddImage, handleOpenAddImage] = useModalHandler();
    const [addAlertGroupOpen, handleCloseAddAlertGroup, handleOpenAddAlertGroup] = useModalHandler();

    const canvasRenderer = React.useMemo(() => {

        return new LayoutEditViewRenderer();

    }, [model.viewModel.updated_at]);

    const setCanvasRef = React.useCallback((e) => {
        canvasRenderer.setCanvasRef(e);
    }, [canvasRenderer]);
    const handleMouseDown = React.useCallback((e) => {
        canvasRenderer.handleMouseDown(e);
    }, [canvasRenderer]);
    const handleMouseMove = React.useCallback((e) => {
        canvasRenderer.handleMouseMove(e);
    }, [canvasRenderer]);
    const handleMouseUp = React.useCallback((e) => {
        canvasRenderer.handleMouseUp(e);
    }, [canvasRenderer]);
    const handleMouseLeave = React.useCallback((e) => {
        canvasRenderer.handleMouseLeave();
    }, [canvasRenderer]);
    const handleKeyDown = React.useCallback((e) => {
        canvasRenderer.handleKeyDown(e);
    }, [canvasRenderer]);

    useEffect(() => {
        if (!canvasRenderer.canvas) return;
        canvasRenderer.setLayout(model.viewModel);
        if (!MachineStore.isLoading) {
            canvasRenderer.draw();
        }
    }, [model.viewModel, MachineStore.isLoading, canvasRenderer, canvasRenderer.canvas, model.viewModel.resolution_x, model.viewModel.resolution_y])

    const getNewComponentModel = React.useCallback((type: LayoutComponentType) => {
        const c = LabLayoutComponentStore.createModel();
        c.component_type = type;
        return c;
    }, []);

    const selectResolutionOptions = React.useMemo(() => {

        const options = LabLayoutResolutionOptions();

        return options.map((option) => {
            return {
                label: `${option.x}x${option.y}`,
                value: option
            };
        });

    }, []);

    const handleSelectResolution = React.useCallback((name, value) => {

        if (value == null) {
            return;
        }

        model.viewModel.resolution_x = value.x;
        model.viewModel.resolution_y = value.y;

    }, [model]);

    return (
        <ModelForm model={model} suppressMakeCopy suppressImportExport maxWidth={'xl'}>
            <ModelInput name={'name'}/>
            <SelectField
                name={'resolution'}
                label={t`Resolution`}
                options={selectResolutionOptions}
                value={model.viewModel.current_resolution}
                onChange={handleSelectResolution}
            />
            <Conditional condition={MachineStore.isLoading}>
                <IfTrue>
                    <LinearProgress/>
                    <Typography variant={'subtitle1'}><Trans>Loading layout editor...</Trans></Typography>
                </IfTrue>
                <IfFalse>
                    <ButtonField startIcon={<MemoryIcon/>} text={t`Add Machine`} onClick={handleOpenAddMachine}/>
                    <ButtonField startIcon={<ImageRounded/>} text={t`Add Image`} onClick={handleOpenAddImage}/>
                    <ButtonField startIcon={<ErrorOutline/>} text={t`Add Alert Group`}
                                 onClick={handleOpenAddAlertGroup}/>
                    <AddStatusLegendButton renderer={canvasRenderer}/>
                    <Grid container spacing={2}>
                        <Grid item xs={10}>
                            <canvas style={{width: '100%'}}
                                    tabIndex={0} //required for onKeyDown to fire
                                    width={model.viewModel.resolution_x}
                                    height={model.viewModel.resolution_y}
                                    className={classes.canvas}
                                    onMouseDown={handleMouseDown}
                                    onMouseMove={handleMouseMove}
                                    ref={setCanvasRef}
                                    onTouchMove={handleMouseMove}
                                    onTouchStart={handleMouseDown}
                                    onTouchEnd={handleMouseUp}
                                    onMouseUp={handleMouseUp}
                                    onMouseLeave={handleMouseLeave}
                                    onKeyDown={handleKeyDown}
                            />
                        </Grid>
                        <Grid item xs={2}>
                            <LayoutConfigComponentDetails renderer={canvasRenderer}/>
                        </Grid>
                    </Grid>
                </IfFalse>
            </Conditional>
            <AddLayoutMachineModal model={getNewComponentModel("machine_status_dot")} open={addMachineOpen}
                                   handleClose={handleCloseAddMachine} renderer={canvasRenderer}/>
            <AddLayoutImageModal model={getNewComponentModel("image")} open={addImageOpen}
                                 handleClose={handleCloseAddImage} renderer={canvasRenderer}/>
            <AddAlertGroupModal model={getNewComponentModel("alert_group")} open={addAlertGroupOpen}
                                handleClose={handleCloseAddAlertGroup} renderer={canvasRenderer}/>
        </ModelForm>
    )
});
